/*
 *  Copyright (C) 2003 Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-action-stop.h"
#include "galeon-shell.h"
#include "gul-gobject-misc.h"
#include "gul-toolbutton.h"
#include "gul-string.h"
#include "gul-gui.h"
#include <gtk/gtkstock.h>
#include <libgnome/gnome-i18n.h>
#include <string.h>

#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
#define DEBUG_MSG(x) g_print x
//#define DEBUG_MSG(x) 


static void		galeon_action_stop_init				(GaleonActionStop *action);
static void		galeon_action_stop_class_init			(GaleonActionStopClass *class);
static void		galeon_action_stop_finalize			(GObject *object);
static void		galeon_action_stop_connect_proxy		(EggAction *action, GtkWidget *proxy);
static void		galeon_action_stop_update			(GaleonAction *action);
static void		galeon_action_stop_current_embed_changed	(GaleonAction *action, 
									 GaleonEmbed *old_embed, GaleonEmbed *new_embed);
static GtkWidget *	galeon_action_stop_create_toolitem		(EggAction *action);
static GtkWidget *	galeon_action_stop_create_menu_item		(EggAction *action);

static EggActionClass *parent_class = NULL;

MAKE_GET_TYPE (galeon_action_stop, "GaleonActionStop", GaleonActionStop,
	       galeon_action_stop_class_init, 
	       galeon_action_stop_init, GALEON_TYPE_ACTION);


static void
galeon_action_stop_class_init (GaleonActionStopClass *class)
{
	GObjectClass *object_class = G_OBJECT_CLASS (class);
	EggActionClass *action_class = EGG_ACTION_CLASS (class);
	GaleonActionClass *galeon_action_class = GALEON_ACTION_CLASS (class);

	object_class->finalize = galeon_action_stop_finalize;

	action_class->create_tool_item = galeon_action_stop_create_toolitem;
	action_class->create_menu_item = galeon_action_stop_create_menu_item;
	action_class->connect_proxy = galeon_action_stop_connect_proxy;

	galeon_action_class->update = galeon_action_stop_update;
	galeon_action_class->current_embed_changed = galeon_action_stop_current_embed_changed;
	
	parent_class = g_type_class_peek_parent (class);
}

static void
galeon_action_stop_init (GaleonActionStop *action)
{
	g_object_set (G_OBJECT (action), "stock_id", GTK_STOCK_STOP, NULL);
	g_object_set (G_OBJECT (action), "tooltip", _("Stop current data transfer"), NULL);
}

static void
galeon_action_stop_finalize (GObject *object)
{
	GaleonAction *action = GALEON_ACTION(object);

	if (action->embed)
		g_signal_handlers_disconnect_matched(action->embed,
						     G_SIGNAL_MATCH_DATA,
						     0, 0, NULL, NULL, action);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
galeon_action_stop_update (GaleonAction *action)
{
	gboolean sensitive;
	GaleonWindow *window = galeon_action_get_window (action);
	GaleonTab *tab = galeon_window_get_active_tab (window);

	if (tab)
	{
		sensitive = galeon_tab_get_load_status (tab) & TAB_LOAD_STARTED;
	}
	else
	{
		sensitive = FALSE;
	}

	g_object_set (G_OBJECT (action), "sensitive", sensitive, NULL);
}

static void
galeon_action_stop_net_stop_start_cb (GaleonEmbed *embed, GaleonAction *action)
{
	galeon_action_stop_update (action);
}

static void
galeon_action_stop_current_embed_changed (GaleonAction *action, 
					  GaleonEmbed *old_embed, GaleonEmbed *new_embed)
{
	if (old_embed)
	{
		g_signal_handlers_disconnect_matched (old_embed, G_SIGNAL_MATCH_DATA,
						      0, 0, NULL, NULL, action);
	}

	if (new_embed)
	{
		g_signal_connect (new_embed, "ge_net_start", 
				  G_CALLBACK (galeon_action_stop_net_stop_start_cb), action);
		g_signal_connect (new_embed, "ge_net_stop", 
				  G_CALLBACK (galeon_action_stop_net_stop_start_cb), action);
	}

	galeon_action_stop_update (action);
}

static void
galeon_action_stop_button_clicked_cb (GtkWidget *w, GaleonActionStop *a)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (a));

	g_return_if_fail (embed != NULL);

	galeon_embed_stop_load (embed);
}

static void
galeon_action_stop_stop_this_tab_activate_cb (GtkMenuItem *mi, GaleonActionStop *b)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (b));
	galeon_embed_stop_load (embed);
}

static void
galeon_action_stop_stop_all_tabs_activate_cb (GtkMenuItem *mi, GaleonActionStop *b)
{
	GList *tabs;
	GaleonWindow *window;
	GList *li;

	window = galeon_action_get_window (GALEON_ACTION (b));
	g_return_if_fail (window != NULL);

	tabs = galeon_window_get_tabs (window);
	for (li = tabs; li; li = li->next)
	{
		GaleonEmbed *embed = galeon_tab_get_embed (li->data);
		galeon_embed_stop_load (embed);
	}
	
	g_list_free (tabs);
}

static void
galeon_action_stop_stop_all_windows_activate_cb (GtkMenuItem *mi, GaleonActionStop *b)
{
	const GList *windows;
	const GList *lj;
	Session *session;

	session = galeon_shell_get_session (galeon_shell);
	windows = session_get_windows (session);

	for (lj = windows; lj; lj = lj->next)
	{
		GaleonWindow *window = lj->data;
		GList *tabs;
		GList *li;
		
		tabs = galeon_window_get_tabs (window);
		for (li = tabs; li; li = li->next)
		{
			GaleonEmbed *embed = galeon_tab_get_embed (li->data);
			galeon_embed_stop_load (embed);
		}
		
		g_list_free (tabs);
	}
}

static GtkWidget *
galeon_action_stop_create_toolitem (EggAction *a)
{
	GulToolbutton *b = gul_toolbutton_new ();
	GtkMenuShell *ms;

	gul_toolbutton_set_show_arrow (b, FALSE);
	gul_toolbutton_set_enable_menu (b, TRUE);
	egg_tool_item_set_homogeneous (EGG_TOOL_ITEM (b), FALSE);
	
	g_signal_connect (b, "clicked",
			  G_CALLBACK (galeon_action_stop_button_clicked_cb), a);

	ms = gul_toolbutton_get_menu (b);
	
	gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("_Stop Loading This Tab"), 
				     G_CALLBACK (galeon_action_stop_stop_this_tab_activate_cb), a);
	gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Stop Loading All Tabs in This _Window"), 
				     G_CALLBACK (galeon_action_stop_stop_all_tabs_activate_cb), a);
	gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Stop Loading _All Tabs"), 
				     G_CALLBACK (galeon_action_stop_stop_all_windows_activate_cb), a);
		
	return GTK_WIDGET (b);
}

static GtkWidget *
galeon_action_stop_create_menu_item (EggAction *a)
{
	GtkWidget *w = gtk_image_menu_item_new ();

	g_signal_connect (w, "activate",
			  G_CALLBACK (galeon_action_stop_button_clicked_cb), a);

	return w;
}

static void
galeon_action_stop_connect_proxy (EggAction *a, GtkWidget *proxy)
{
	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (a, proxy);
}

