/*
 *  Copyright (C) 2002  Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <libgnome/gnome-i18n.h>
#include <gtk/gtkradiomenuitem.h>
#include <gtk/gtktearoffmenuitem.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtkimage.h>
#include "galeon-recent-history-menu.h"
#include "gul-gobject-misc.h"
#include "galeon-marshal.h"
#include "gul-string.h"
#include "gul-gui.h"

#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
//#define DEBUG_MSG(x) g_print x
#define DEBUG_MSG(x)

/**
 * Private data
 */
struct _GaleonRecentHistoryMenuPrivate {
	GaleonWindowRecentHistory *wrh;
	GtkMenuShell *ms;
	guint rebuild_timeout;

	GSList *menuitems;
	GSList *urls;

	gint ignore_menu_callbacks;

	GHashTable *menuitem_to_url;

	GtkWidget *parent_widget;
	GaleonWindow *window;
};

#define REBUILD_TIMEOUT 700

/**
 * Private functions, only availble from this file
 */
static void		galeon_recent_history_menu_class_init		(GaleonRecentHistoryMenuClass *klass);
static void		galeon_recent_history_menu_init			(GaleonRecentHistoryMenu *e);
static void		galeon_recent_history_menu_finalize_impl	(GObject *o);
static void		galeon_recent_history_menu_build_submenu	(GaleonRecentHistoryMenu *gm);
static void		galeon_recent_history_menu_build_item		(GaleonRecentHistoryMenu *gm, 
									 gchar *url, const gchar *title, gint i);

static void		galeon_recent_history_menu_rebuild		(GaleonRecentHistoryMenu *gm);
static void 		galeon_recent_history_menu_rebuild_with_timeout	(GaleonRecentHistoryMenu *gm);
static gboolean		galeon_recent_history_menu_rebuild_timeout_cb	(gpointer data);
static void		galeon_recent_history_menu_activate_cb		(GtkMenuItem *mi, GaleonRecentHistoryMenu *cm);

static gpointer g_object_class;

/**
 * GtkMenu object
 */

MAKE_GET_TYPE (galeon_recent_history_menu, "GaleonRecentHistoryMenu", GaleonRecentHistoryMenu,
	       galeon_recent_history_menu_class_init,
	       galeon_recent_history_menu_init, G_TYPE_OBJECT);

static void
galeon_recent_history_menu_class_init (GaleonRecentHistoryMenuClass *klass)
{
	G_OBJECT_CLASS (klass)->finalize = galeon_recent_history_menu_finalize_impl;
	g_object_class = g_type_class_peek_parent (klass);

}

static void 
galeon_recent_history_menu_init (GaleonRecentHistoryMenu *m)
{
	GaleonRecentHistoryMenuPrivate *p = g_new0 (GaleonRecentHistoryMenuPrivate, 1);
	m->priv = p;
	
	p->menuitem_to_url = g_hash_table_new (NULL, NULL);
	p->ignore_menu_callbacks = 0;
	p->parent_widget = NULL;
	p->window = NULL;
}

static void
galeon_recent_history_menu_finalize_impl (GObject *o)
{
	GaleonRecentHistoryMenu *gm = GALEON_RECENT_HISTORY_MENU (o);
	GaleonRecentHistoryMenuPrivate *p = gm->priv;
	GSList *li;

	DEBUG_MSG (("Finalizing GaleonRecentHistoryMenu\n"));

	galeon_recent_history_menu_set_parent_widget (gm, NULL);
	galeon_recent_history_menu_set_window (gm, NULL);
	
	if (p->wrh)
	{
		g_signal_handlers_disconnect_matched (p->wrh, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gm);
		g_object_unref (p->wrh);
	}

	if (p->rebuild_timeout)
	{
		g_source_remove (p->rebuild_timeout);
	}

	for (li = p->menuitems; li; li = li->next)
	{
		g_object_unref (li->data);
	}
	g_slist_free (p->menuitems);

	g_slist_foreach (p->urls, (GFunc) g_free, NULL);
	g_slist_free (p->urls);
	p->urls = NULL;

	if (p->ms)
	{
		g_object_unref (p->ms);
	}
	g_hash_table_destroy (p->menuitem_to_url);

	g_free (p);

	G_OBJECT_CLASS (g_object_class)->finalize (o);
}

GaleonRecentHistoryMenu *
galeon_recent_history_menu_new (GtkMenuShell *ms)
{
	GaleonRecentHistoryMenu *ret = g_object_new (GALEON_TYPE_RECENT_HISTORY_MENU, NULL);
	GaleonRecentHistoryMenuPrivate *p = ret->priv;

	p->wrh = NULL;

	p->ms = g_object_ref (ms);

	p->menuitems = NULL;
	p->urls = NULL;
	galeon_recent_history_menu_rebuild_with_timeout (ret);

	return ret;
}

static void
galeon_recent_history_menu_build_submenu (GaleonRecentHistoryMenu *gm)
{
	GaleonRecentHistoryMenuPrivate *p = gm->priv;
	gint i;
	gint num_items;

	p->ignore_menu_callbacks++;

	gul_gui_setup_tearoff (p->ms);

	g_assert (p->urls == NULL);

	galeon_window_recent_history_get_num_items (p->wrh, &num_items);
	for (i = 0; i < num_items; ++i)
	{
		gchar *title;
		gchar *url;

		galeon_window_recent_history_get_item (p->wrh, i, &url, &title);
		galeon_recent_history_menu_build_item (gm, url, title, i);
		g_free (title);
		/* the url does not have to be freed, because we keep a reference to it */
	}

	if (GTK_IS_WIDGET (p->parent_widget))
	{
		if (p->urls)
		{
			gtk_widget_set_sensitive (p->parent_widget, TRUE);
		}
		else
		{
			gtk_widget_set_sensitive (p->parent_widget, FALSE);
		}
	}
	p->ignore_menu_callbacks--;
}

#define MENU_ITEM_MAX_LENGTH 40

static void
galeon_recent_history_menu_build_item (GaleonRecentHistoryMenu *gm, gchar *url, const gchar *title, gint i)
{
	GaleonRecentHistoryMenuPrivate *p = gm->priv;
	GtkWidget *w;
	gchar *title_s = gul_string_shorten (title, MENU_ITEM_MAX_LENGTH);
	gchar *label = gul_string_new_num_accel (i, title_s, TRUE);

	p->ignore_menu_callbacks++;

	w = gtk_menu_item_new_with_mnemonic (label);

	g_signal_connect (w, "activate", 
			  G_CALLBACK (galeon_recent_history_menu_activate_cb), gm);

	p->menuitems = g_slist_prepend (p->menuitems, g_object_ref (w));
	p->urls = g_slist_prepend (p->urls, url);
	g_hash_table_insert (p->menuitem_to_url, w, url);

	gtk_widget_show (w);
	gtk_container_add (GTK_CONTAINER (p->ms), w);

	g_free (title_s);
	g_free (label);

	p->ignore_menu_callbacks--;
}

static void
galeon_recent_history_menu_activate_cb (GtkMenuItem *mi, GaleonRecentHistoryMenu *cm)
{
	GaleonRecentHistoryMenuPrivate *p = cm->priv;
	gchar *url;
	
	if (p->ignore_menu_callbacks > 0)
	{
		return;
	}

	DEBUG_MSG (("galeon_recent_history_menu_activated_cb\n"));

	url = g_hash_table_lookup (p->menuitem_to_url, mi);
	if (url)
	{
		DEBUG_MSG (("Going to '%s'\n", url));
		
		if (p->window)
		{
			galeon_window_load_url (p->window, url);
		}
	}
	else
	{
		DEBUG_MSG (("NULL url assoicated to menu item\n"));
	}
}

static gboolean
galeon_recent_history_menu_rebuild_timeout_cb (gpointer data)
{
	GaleonRecentHistoryMenu *gm = data;
	GaleonRecentHistoryMenuPrivate *p = gm->priv;
	DEBUG_MSG (("GaleonRecentHistoryMenu rebuild timeout\n"));

	p->rebuild_timeout = 0;

	galeon_recent_history_menu_rebuild (data);
	return FALSE;
}

static void 
galeon_recent_history_menu_rebuild_with_timeout (GaleonRecentHistoryMenu *gm) 
{
	GaleonRecentHistoryMenuPrivate *p = gm->priv;

	DEBUG_MSG (("galeon_recent_history_menu_rebuild_with_timeout\n"));

	if (p->rebuild_timeout)
	{
		g_source_remove (p->rebuild_timeout);
	}
	
	p->rebuild_timeout = g_timeout_add (REBUILD_TIMEOUT, 
					    galeon_recent_history_menu_rebuild_timeout_cb, gm);
}

static void
galeon_recent_history_menu_reset (GaleonRecentHistoryMenu *gm)
{
	GaleonRecentHistoryMenuPrivate *p = gm->priv;
	GSList *sli;

	p->ignore_menu_callbacks++;

	/* free the list of menuitems */
	for (sli = p->menuitems; sli; sli = sli->next)
	{
		g_assert (GTK_IS_WIDGET (sli->data) && GTK_WIDGET (sli->data)->parent == GTK_WIDGET (p->ms));
		gtk_container_remove (GTK_CONTAINER (p->ms), sli->data);
	}
	for (sli = p->menuitems; sli; sli = sli->next)
	{
		g_signal_handlers_disconnect_matched (sli->data, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gm);
		g_object_unref (sli->data);
	}
	g_slist_free (p->menuitems);
	p->menuitems = NULL;

	g_slist_foreach (p->urls, (GFunc) g_free, NULL);
	g_slist_free (p->urls);
	p->urls = NULL;

	if (g_hash_table_size (p->menuitem_to_url) > 0)
	{
		g_hash_table_destroy (p->menuitem_to_url);
		p->menuitem_to_url = g_hash_table_new (NULL, NULL);
	}

	p->ignore_menu_callbacks--;
}

static void
galeon_recent_history_menu_rebuild (GaleonRecentHistoryMenu *gm)
{
	GaleonRecentHistoryMenuPrivate *p = gm->priv;

	DEBUG_MSG (("galeon_recent_history_menu_rebuild called\n"));

	p->ignore_menu_callbacks++;

	galeon_recent_history_menu_reset (gm);
	
	galeon_recent_history_menu_build_submenu (gm);

	p->ignore_menu_callbacks--;
}


static void
galeon_recent_history_menu_changed_cb (GaleonWindowRecentHistory *wrh, 
				       GaleonRecentHistoryMenu *cm)
{
	GaleonRecentHistoryMenuPrivate *p = cm->priv;

	g_return_if_fail (wrh == p->wrh);

	galeon_recent_history_menu_rebuild_with_timeout (cm);
}

void 
galeon_recent_history_menu_set_history (GaleonRecentHistoryMenu *cm, GaleonWindowRecentHistory *wrh)
{
	GaleonRecentHistoryMenuPrivate *p;

	g_return_if_fail (GALEON_IS_RECENT_HISTORY_MENU (cm));
	g_return_if_fail (!wrh || GALEON_IS_WINDOW_RECENT_HISTORY (wrh));

	DEBUG_MSG (("galeon_recent_history_menu_set_history\n"));

	p = cm->priv;

	if (p->wrh)
	{
		g_signal_handlers_disconnect_matched (p->wrh, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, cm);
		g_object_unref (p->wrh);
		p->wrh = NULL;
	}

	if (wrh)
	{
		p->wrh = g_object_ref (wrh);
		g_signal_connect (p->wrh, "changed",
				  G_CALLBACK (galeon_recent_history_menu_changed_cb), cm);
	}

	galeon_recent_history_menu_rebuild_with_timeout (cm);
}

void
galeon_recent_history_menu_set_parent_widget (GaleonRecentHistoryMenu *cm, GtkWidget *w)
{
	GaleonRecentHistoryMenuPrivate *p = cm->priv;
	
	if (p->parent_widget)
	{
		g_object_remove_weak_pointer (G_OBJECT (p->parent_widget),
					      (gpointer *) &p->parent_widget);
	}

	p->parent_widget = w;

	if (p->parent_widget)
	{
		g_object_add_weak_pointer (G_OBJECT (p->parent_widget), 
					   (gpointer *) &p->parent_widget);
	}
}

void
galeon_recent_history_menu_set_window (GaleonRecentHistoryMenu *cm,
				       GaleonWindow *w)
{
	GaleonRecentHistoryMenuPrivate *p = cm->priv;
	
	if (p->window)
	{
		g_object_remove_weak_pointer (G_OBJECT (p->window),
					      (gpointer *) &p->window);
	}

	p->window = w;

	if (p->window)
	{
		g_object_add_weak_pointer (G_OBJECT (p->window), 
					   (gpointer *) &p->window);
	}
}

