/*
 *  Copyright (C) 2003  Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gul-toolbutton.h"
#include "gul-gobject-misc.h"
#include "gul-gui.h"
#include "galeon-marshal.h"

#include <gtk/gtkarrow.h>
#include <gtk/gtkmain.h>
#include <gdk/gdkkeysyms.h>
#include <libgnome/gnome-i18n.h>

#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
//#define DEBUG_MSG(x) g_print x
#define DEBUG_MSG(x)

/**
 * Private data
 */
struct _GulToolbuttonPrivate
{
	GtkMenu *menu;
	GtkWidget *arrow_widget;
	GtkWidget *button;

	gboolean show_arrow;
};

static void		gul_toolbutton_init			(GulToolbutton *button);
static void		gul_toolbutton_class_init		(GulToolbuttonClass *klass);
static void		gul_toolbutton_finalize_impl		(GObject *o);
static void		gul_toolbutton_build			(GulToolbutton *button);
static gboolean		gul_toolbutton_set_tooltip		(EggToolItem *tool_item,
								 GtkTooltips *tooltips,
								 const gchar *tip_text,
								 const gchar *tip_private);

enum GulToolbuttonSignalsEnum {
	GUL_TOOLBUTTON_MENU_ACTIVATED,
	GUL_TOOLBUTTON_LAST_SIGNAL
};
static gint GulToolbuttonSignals[GUL_TOOLBUTTON_LAST_SIGNAL];

static GObjectClass *parent_class = NULL;


MAKE_GET_TYPE (gul_toolbutton, "GulToolbutton", GulToolbutton, gul_toolbutton_class_init, 
	       gul_toolbutton_init, EGG_TYPE_TOOL_BUTTON);



static void
gul_toolbutton_class_init (GulToolbuttonClass *klass)
{
	GObjectClass *object_class;
	EggToolItemClass *toolitem_class;
	
	parent_class = g_type_class_peek_parent (klass);
	
	object_class = (GObjectClass *) klass;
	toolitem_class = (EggToolItemClass *) klass;

	object_class->finalize = gul_toolbutton_finalize_impl;
	
	toolitem_class->set_tooltip = gul_toolbutton_set_tooltip;

	GulToolbuttonSignals[GUL_TOOLBUTTON_MENU_ACTIVATED] = g_signal_new (
		"menu-activated", G_OBJECT_CLASS_TYPE (klass),  
		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
		G_STRUCT_OFFSET (GulToolbuttonClass, menu_activated), 
		NULL, NULL, 
		galeon_marshal_VOID__VOID,
		G_TYPE_NONE, 0);
}

static void
gul_toolbutton_init (GulToolbutton *button)
{
	GulToolbuttonPrivate *p = g_new0 (GulToolbuttonPrivate, 1);
	button->priv = p;
	
	p->show_arrow = FALSE;

	gul_toolbutton_build (button);
}

static void
gul_toolbutton_finalize_impl (GObject *o)
{
	GulToolbutton *b = GUL_TOOLBUTTON (o);
	GulToolbuttonPrivate *p = b->priv;

	if (p->button)
	{
		g_object_unref (p->button);
	}

	if (p->menu)
	{
		g_object_unref (p->menu);
	}

	if (p->arrow_widget)
	{
		g_object_unref (p->arrow_widget);
	}

	g_free (p);
	
	DEBUG_MSG (("GulToolbutton finalized\n"));
	
	G_OBJECT_CLASS (parent_class)->finalize (o);
}

static void 
gul_toolbutton_popup_menu_under_arrow (GulToolbutton *b, GdkEventButton *event)
{
	GulToolbuttonPrivate *p = b->priv;

	if (p->menu)
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (p->arrow_widget), TRUE);
		g_signal_emit (b, GulToolbuttonSignals[GUL_TOOLBUTTON_MENU_ACTIVATED], 0);
		gtk_menu_popup (p->menu, NULL, NULL, gul_gui_menu_position_under_widget, p->arrow_widget, 
				event ? event->button : 0, 
				event ? event->time : gtk_get_current_event_time ());
	}
}

static gboolean
gul_toolbutton_arrow_button_press_event_cb  (GtkWidget *widget, GdkEventButton *event, GulToolbutton *b)
{
	gul_toolbutton_popup_menu_under_arrow (b, event);
	return TRUE;
}

static gboolean
gul_toolbutton_arrow_key_press_event_cb (GtkWidget *widget, GdkEventKey *event, GulToolbutton *b)
{
	if (event->keyval == GDK_space
	    || event->keyval == GDK_KP_Space
	    || event->keyval == GDK_Return
	    || event->keyval == GDK_KP_Enter
	    || event->keyval == GDK_Menu)
	{
		gul_toolbutton_popup_menu_under_arrow (b, NULL);
	}

	return FALSE;
}

static gboolean
gul_toolbutton_button_button_press_event_cb (GtkWidget *widget, GdkEventButton *event, 
					     GulToolbutton *b)
{
	GulToolbuttonPrivate *p = b->priv;

	if (event->button == 3 && p->menu)
	{
		g_signal_emit (b, GulToolbuttonSignals[GUL_TOOLBUTTON_MENU_ACTIVATED], 0);
		gtk_menu_popup (p->menu, NULL, NULL, NULL, b, 
				event ? event->button : 0, 
				event ? event->time : gtk_get_current_event_time ());
		return TRUE;
	}
	
	return FALSE;
}

static void
gul_toolbutton_state_changed_cb (GtkWidget *widget,
				 GtkStateType previous_state,
				 GulToolbutton *b)
{
	GulToolbuttonPrivate *p = b->priv;
	GtkWidget *button;
	GtkStateType state = GTK_WIDGET_STATE (widget);

	if (!p->arrow_widget) return;

	button = (widget == p->arrow_widget) ? p->button : p->arrow_widget;

	g_signal_handlers_block_by_func
		         (G_OBJECT (button),
		          G_CALLBACK (gul_toolbutton_state_changed_cb), b);
	
	if (state == GTK_STATE_PRELIGHT &&
	    previous_state != GTK_STATE_ACTIVE)
	{
		gtk_widget_set_state (button, state);
	}
	else if (state == GTK_STATE_NORMAL)
	{
		 gtk_widget_set_state (button, state);
	}
	else if (state == GTK_STATE_ACTIVE)
	{
		gtk_widget_set_state (button, GTK_STATE_NORMAL);
	}

	g_signal_handlers_unblock_by_func
		         (G_OBJECT (button),
		          G_CALLBACK (gul_toolbutton_state_changed_cb), b);
}

static void 
gul_toolbutton_button_popup_menu_cb (GtkWidget *w, GulToolbutton *b)
{
	GulToolbuttonPrivate *p = b->priv;

	if (p->menu)
	{
		g_signal_emit (b, GulToolbuttonSignals[GUL_TOOLBUTTON_MENU_ACTIVATED], 0);
		gtk_menu_popup (p->menu, NULL, NULL, 
				gul_gui_menu_position_under_widget, b, 0, gtk_get_current_event_time ());
	}
}

static void
gul_toolbutton_build (GulToolbutton *button)
{
	GulToolbuttonPrivate *p = button->priv;
	GtkWidget *hbox;

	if (!p->button)
	{
		p->button = g_object_ref (GTK_BIN (button)->child);
		gtk_container_remove (GTK_CONTAINER (button), p->button);
		
		hbox = gtk_hbox_new (FALSE, 0);
		gtk_container_add (GTK_CONTAINER (button), hbox);
		gtk_widget_show (hbox);
		gtk_box_pack_start_defaults (GTK_BOX (hbox), p->button);

		g_signal_connect (p->button, "button_press_event",
				  G_CALLBACK (gul_toolbutton_button_button_press_event_cb), button);
		g_signal_connect (p->button, "popup_menu", 
				  G_CALLBACK (gul_toolbutton_button_popup_menu_cb), button);
		g_signal_connect (p->button, "state_changed",
				  G_CALLBACK (gul_toolbutton_state_changed_cb), button);
	}
	else
	{
		hbox = GTK_BIN (button)->child;
	}

	if (p->show_arrow)
	{
		if (!p->arrow_widget)
		{
			p->arrow_widget = gtk_toggle_button_new ();
			g_object_ref (p->arrow_widget);
			gtk_object_sink (GTK_OBJECT (p->arrow_widget));

			gtk_button_set_relief (GTK_BUTTON (p->arrow_widget), GTK_RELIEF_NONE);
		
			gtk_container_add (GTK_CONTAINER (p->arrow_widget),
					   gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT));
		
			g_signal_connect (p->arrow_widget, "key_press_event",
					  G_CALLBACK (gul_toolbutton_arrow_key_press_event_cb),
					  button);
			g_signal_connect (p->arrow_widget, "button_press_event",
					  G_CALLBACK (gul_toolbutton_arrow_button_press_event_cb),
					  button);
			g_signal_connect (p->arrow_widget, "state_changed",
					  G_CALLBACK (gul_toolbutton_state_changed_cb),
					  button);
			
			gtk_widget_show_all (p->arrow_widget);
			gtk_box_pack_end_defaults (GTK_BOX (hbox), p->arrow_widget);
		}
	}
	else
	{
		if (p->arrow_widget)
		{
			if (p->arrow_widget->parent)
			{
				gtk_container_remove (GTK_CONTAINER (p->arrow_widget->parent), p->arrow_widget);
			}
			g_object_unref (p->arrow_widget);
			p->arrow_widget = NULL;
		}
	}
}

GulToolbutton *
gul_toolbutton_new (void)
{
	GulToolbutton *ret = g_object_new (GUL_TYPE_TOOLBUTTON, NULL);
	
	return ret;
}

GulToolbutton *
gul_toolbutton_new_from_stock (const gchar *stock_id)
{
	GulToolbutton *ret;
	
	g_return_val_if_fail (stock_id != NULL, NULL);
	
	ret = g_object_new (GUL_TYPE_TOOLBUTTON,
			    "stock_id", stock_id,
			    NULL);
  	return ret;
}

void
gul_toolbutton_set_show_arrow (GulToolbutton *b, gboolean value)
{
	GulToolbuttonPrivate *p = b->priv;
	value = !!value;
	if (value != p->show_arrow)
	{
		p->show_arrow = value;
		gul_toolbutton_build (b);
	}
}

static void 
gul_toolbutton_menu_deactivated_cb (GtkMenuShell *ms, GulToolbutton *b)
{
	GulToolbuttonPrivate *p = b->priv;
	if (p->arrow_widget)
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (p->arrow_widget), FALSE);
	}
}

void
gul_toolbutton_set_enable_menu (GulToolbutton *b, gboolean value)
{
	GulToolbuttonPrivate *p = b->priv;
	if (value)
	{
		p->menu = GTK_MENU (gtk_menu_new ());
		g_object_ref (p->menu);
		gtk_object_sink (GTK_OBJECT (p->menu));
		g_signal_connect (p->menu, "deactivate", 
				  G_CALLBACK (gul_toolbutton_menu_deactivated_cb), b);

	}
	else
	{
		if (p->menu)
		{
			g_object_unref (p->menu);
			p->menu = NULL;
		}
	}
}

GtkMenuShell *
gul_toolbutton_get_menu (GulToolbutton *b)
{
	GulToolbuttonPrivate *p = b->priv;
	return GTK_MENU_SHELL (p->menu);	
}

GtkWidget *
gul_toolbutton_get_button (GulToolbutton *b)
{
	GulToolbuttonPrivate *p = b->priv;
	
	if (!p->button)
	{
		gul_toolbutton_build (b);
	}

	return p->button;
}

static gboolean
gul_toolbutton_set_tooltip (EggToolItem *tool_item,
			    GtkTooltips *tooltips,
			    const gchar *tip_text,
			    const gchar *tip_private)
{
	GulToolbuttonPrivate *p = GUL_TOOLBUTTON (tool_item)->priv;
	GtkWidget *button = gul_toolbutton_get_button (GUL_TOOLBUTTON (tool_item));
	
	gtk_tooltips_set_tip (tooltips, button, tip_text, tip_private);
	if (p->arrow_widget)
	{
		gtk_tooltips_set_tip (tooltips, p->arrow_widget, tip_text, tip_private);
	}
	
	return TRUE;
}
